/* ========================================
 *
 * Simple demonstration of reading 2 8-bit signals as a 16-bit word
 * from 2 UDBs using the PI parallel inputs
 *
 * Can't use cy_psoc3_dp16 since it doesn't allow PI input
 * so have to manually position the UDBs using directives:
 *   udb_1:test ForceComponentUDB U(0,0)
 *   udb_2:test ForceComponentUDB U(1,0)
 * 
 * Tested on CY8CKIT-059 (PSOC 5LP)
 * Both CPU and DMA reads are performed, result is printed
 * on the Kitprog serial interface
 *
 * Jeremy Bentham 29 March 2019
 * 
 * ========================================
*/
#include "project.h"
#include <stdio.h>
#include <stdbool.h>
#include <stdarg.h>

#define UDB_1_A0 udb_1_test__A0_REG
#define UDB_2_A0 udb_2_test__A0_REG

uint16 dma_val;
uint8 dma_chan, dma_td, printbuff[1000];

int print(const char *fmt, ...);

int main(void)
{
    CyGlobalIntEnable; /* Enable global interrupts. */
    UART_1_Start();
    print("UDB test\r\n");
    
    dma_chan = DMA_1_DmaInitialize(2, 1, HI16(CYDEV_PERIPH_BASE), HI16(CYDEV_SRAM_BASE));
    dma_td = CyDmaTdAllocate();
    CyDmaTdSetConfiguration(dma_td, 2, DMA_DISABLE_TD, TD_INC_SRC_ADR|TD_INC_DST_ADR);
    CyDmaTdSetAddress(dma_td, LO16((uint32)UDB_1_A0), LO16((uint32)&dma_val));
    CyDmaChSetInitialTd(dma_chan, dma_td);
    CyDmaChEnable(dma_chan, 1);
    CyDmaChSetRequest(dma_chan, CY_DMA_CPU_REQ);
    
    print("CPU read = %04X, DMA read = %04X\r\n", CY_GET_REG16(UDB_1_A0), dma_val);

    for(;;)
    {
        
    }
}

// printf function for diagnostic output
int print(const char *fmt, ...)
{
    int ret;
    va_list ap;
    va_start(ap, fmt);
    ret = vsnprintf((char *)printbuff, sizeof(printbuff), fmt, ap);
    va_end(ap);
    UART_1_PutArray(printbuff, ret);
    return(ret);
}

/* [] END OF FILE */
